@@ -6,6 +6,7 @@ from conan.cli.command import conan_command, OnceArgument, conan_subcommand
 from conan.cli import make_abs_path
 from conan.cli.args import common_graph_args, validate_common_graph_args
 from conan.cli.printers.graph import print_graph_packages, print_graph_basic
+from conan.errors import ConanException
 from conan.api.model import RecipeReference
 
 
@@ -25,7 +26,6 @@ def lock_create(conan_api, parser, subparser, *args):
     subparser.add_argument("--build-require", action='store_true', default=False,
                            help='Whether the provided reference is a build-require')
     args = parser.parse_args(*args)
-
     # parameter validation
     validate_common_graph_args(args)
 
@@ -100,8 +100,7 @@ def lock_add(conan_api, parser, subparser, *args):
 
     lockfile = conan_api.lockfile.get_lockfile(lockfile=args.lockfile, partial=True)
 
-    global_conf = conan_api.config.global_conf
-    allow_uppercase = global_conf.get("core:allow_uppercase_pkg_names", check_type=bool)
+    allow_uppercase = conan_api.config.get("core:allow_uppercase_pkg_names", check_type=bool)
 
     def _parse_requires(reqs):
         if reqs:
@@ -172,3 +171,101 @@ def lock_update(conan_api, parser, subparser, *args):
     lockfile.update(requires=args.requires, build_requires=args.build_requires,
                     python_requires=args.python_requires, config_requires=args.config_requires)
     conan_api.lockfile.save_lockfile(lockfile, args.lockfile_out)
+
+
+@conan_subcommand()
+def lock_upgrade(conan_api, parser, subparser, *args):
+    """
+    (Experimental) Upgrade requires, build-requires or python-requires from an existing lockfile
+    given a conanfile or a reference.
+    """
+    common_graph_args(subparser)
+    subparser.add_argument('--update-requires', action="append",
+                           help='Update requires from lockfile')
+    subparser.add_argument('--update-build-requires', action="append",
+                           help='Update build-requires from lockfile')
+    subparser.add_argument('--update-python-requires', action="append",
+                           help='Update python-requires from lockfile')
+    subparser.add_argument('--build-require', action='store_true', default=False,
+                           help='Whether the provided reference is a build-require')
+    args = parser.parse_args(*args)
+    # parameter validation
+    validate_common_graph_args(args)
+
+    if not any([args.update_requires, args.update_build_requires, args.update_python_requires]):
+        raise ConanException("At least one of --update-requires, --update-build-requires, "
+                             "--update-python-requires should be specified")
+
+    cwd = os.getcwd()
+    path = conan_api.local.get_conanfile_path(args.path, cwd, py=None) if args.path else None
+    remotes = conan_api.remotes.list(args.remote) if not args.no_remote else []
+    overrides = eval(args.lockfile_overrides) if args.lockfile_overrides else None
+    lockfile = conan_api.lockfile.get_lockfile(lockfile=args.lockfile, conanfile_path=path,
+                                               cwd=cwd, partial=True, overrides=overrides)
+    if lockfile is None:
+        raise ConanException("No lockfile specified and default conan.lock not found")
+    profile_host, profile_build = conan_api.profiles.get_profiles_from_args(args)
+
+    # Remove the lockfile entries that will be updated
+    lockfile = conan_api.lockfile.remove_lockfile(lockfile,
+                                                  requires=args.update_requires,
+                                                  python_requires=args.update_python_requires,
+                                                  build_requires=args.update_build_requires)
+    # Resolve new graph
+    if path:
+        graph = conan_api.graph.load_graph_consumer(path, args.name, args.version,
+                                                    args.user, args.channel,
+                                                    profile_host, profile_build, lockfile,
+                                                    remotes, args.update,
+                                                    is_build_require=args.build_require)
+    else:
+        graph = conan_api.graph.load_graph_requires(args.requires, args.tool_requires,
+                                                    profile_host, profile_build, lockfile,
+                                                    remotes, args.update)
+    print_graph_basic(graph)
+    graph.report_graph_error()
+    conan_api.graph.analyze_binaries(graph, args.build, remotes=remotes, update=args.update,
+                                     lockfile=lockfile)
+    print_graph_packages(graph)
+
+    lockfile = conan_api.lockfile.update_lockfile(lockfile, graph, clean=args.lockfile_clean)
+    conan_api.lockfile.save_lockfile(lockfile, args.lockfile_out or "conan.lock")
+
+
+@conan_subcommand()
+def lock_upgrade_config(conan_api, parser, subparser, *args):
+    """
+    (Experimental) Upgrade config requires in a lockfile
+    """
+    common_graph_args(subparser)
+    subparser.add_argument('--update-config-requires', action="append",
+                           help='Update config-requires from lockfile')
+    args = parser.parse_args(*args)
+    validate_common_graph_args(args)
+
+    if not args.update_config_requires:
+        raise ConanException("At least one --update-config-requires should be specified")
+
+    cwd = os.getcwd()
+    path = conan_api.local.get_conanfile_path(args.path, cwd, py=None) if args.path else None
+    remotes = conan_api.remotes.list(args.remote) if not args.no_remote else []
+    lockfile = conan_api.lockfile.get_lockfile(lockfile=args.lockfile, conanfile_path=path,
+                                               cwd=cwd, partial=True)
+    if lockfile is None:
+        raise ConanException("No lockfile specified and default conan.lock not found")
+    profile_host, profile_build = conan_api.profiles.get_profiles_from_args(args)
+
+    # Remove the lockfile entries that will be updated
+    lockfile = conan_api.lockfile.remove_lockfile(lockfile,
+                                                  config_requires=args.update_config_requires)
+
+    if args.path:
+        path = make_abs_path(args.path)
+        reqs, remotes = conan_api.config.load_conanconfig(path, remotes)
+    else:
+        reqs = [RecipeReference.loads(r) for r in args.requires]
+
+    pkgs = conan_api.config.fetch_packages(reqs, lockfile, remotes, profile_host)
+    refs = [p.ref for p in pkgs]
+    lockfile = conan_api.lockfile.add_lockfile(lockfile, config_requires=refs)
+    conan_api.lockfile.save_lockfile(lockfile, args.lockfile_out or "conan.lock")
